@@ -35,7 +35,7 @@ sub pre_bind
     my $conf = $self->{_config};
 
     foreach my $rule (@{$conf->{rewrite}}) {
-        if ($rule->{context} eq 'request') {
+        if ($rule->{context} eq 'request' || $rule->{context} eq 'bindRequest') {
             if (defined($rule->{dn}) && ${$binddnp} !~ /$rule->{dn}/i) {
                 next;
             }
@@ -68,7 +68,7 @@ sub pre_compare
     my $conf = $self->{_config};
 
     foreach my $rule (@{$conf->{rewrite}}) {
-        if ($rule->{context} eq 'request') {
+        if ($rule->{context} eq 'request' || $rule->{context} eq 'compareRequest') {
             if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
                 next;
             }
@@ -109,7 +109,7 @@ sub pre_search
     my $conf = $self->{_config};
 
     foreach my $rule (@{$conf->{rewrite}}) {
-        if ($rule->{context} eq 'request') {
+        if ($rule->{context} eq 'request' || $rule->{context} eq 'searchRequest') {
             if (defined($rule->{dn}) && ${$basep} !~ /$rule->{dn}/i) {
                 next;
             }
@@ -152,18 +152,19 @@ sub post_search
     foreach my $rule (@{$conf->{rewrite}}) {
         if ($rule->{context} eq 'searchResult') {
             for (my $i = 0; $i < @{$entriesp}; $i++) {
-                my %rwcache;
-                my $rc = 0;
                 my $entryStr = encode('utf8', ${$entriesp}[$i]);
-                my $substitution = $rule->{substitution};
-                $substitution =~ s/%0/$entryStr/;
-
                 my (@line) = split(/\n/, $entryStr);
                 my ($dn) = ($line[0] =~ /^dn: (.*)$/);
                 if (defined($rule->{dn}) && $dn !~ /$rule->{dn}/i) {
                     next;
                 }
 
+                my %rwcache;
+                my $rc = 0;
+
+                my $substitution = $rule->{substitution};
+                $substitution =~ s/%0/$entryStr/;
+
                 my $str = $self->_rewriteParse($rule->{match}, $substitution, $dn, \%rwcache);
                 if (!$str) {
                     $self->log(level => 'error', message => "search result rewrite \"$dn\" failed");
@@ -207,7 +208,7 @@ sub pre_modify
     my $conf = $self->{_config};
 
     foreach my $rule (@{$conf->{rewrite}}) {
-        if ($rule->{context} eq 'request') {
+        if ($rule->{context} eq 'request' || $rule->{context} eq 'modifyRequest') {
             if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
                 next;
             }
@@ -245,7 +246,7 @@ sub pre_modify
                             return -1;
                         } elsif ($str) {
                             foreach my $line (split(/\n/, $str)) {
-                                my ($rwattr, $value) = ($line =~ /(^[^:]*): *(.*)$/);
+                                my ($rwattr, $value) = split(/: */, $line);
                                 if (!defined($replaced{$rwattr})) {
                                     @{$replaced{$rwattr}} = ();
                                 }
@@ -282,7 +283,7 @@ sub pre_add
     my $conf = $self->{_config};
 
     foreach my $rule (@{$conf->{rewrite}}) {
-        if ($rule->{context} eq 'request') {
+        if ($rule->{context} eq 'request' || $rule->{context} eq 'addRequest') {
             if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
                 next;
             }
@@ -327,7 +328,7 @@ sub pre_modrdn
     my $conf = $self->{_config};
 
     foreach my $rule (@{$conf->{rewrite}}) {
-        if ($rule->{context} eq 'request') {
+        if ($rule->{context} eq 'request' || $rule->{context} eq 'modrdnRequest') {
             if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
                 next;
             }
@@ -370,7 +371,7 @@ sub pre_delete
     my $conf = $self->{_config};
 
     foreach my $rule (@{$conf->{rewrite}}) {
-        if ($rule->{context} eq 'request') {
+        if ($rule->{context} eq 'request' || $rule->{context} eq 'deleteRequest') {
             if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
                 next;
             }
@@ -524,15 +525,6 @@ sub _ldapMap
     my $rc;
     my @values = ('');
 
-    if (defined($ldapmap->{ldap})) {
-        my $cmsg = $ldapmap->{ldap}->bind($ldapmap->{binddn}, password => $ldapmap->{bindpw});
-        if ($cmsg->code) {
-            $self->log(level => 'error', message => "LDAP connection check failed in handler");
-            $ldapmap->{ldap}->unbind();
-            undef($ldapmap->{ldap});
-        }
-    }
-
     if (!$ldapmap->{ldap}) {
         $ldapmap->{ldap} = Net::LDAP->new($ldapmap->{uri});
         if (!defined($ldapmap->{ldap})) {
