@@ -16,6 +16,17 @@ This class implements the L<LISM::Hanlder> interface to do script.
 
 =head1 METHODS
 
+=head2 getOrder
+
+Get order to do handler.
+
+=cut
+
+sub getOrder
+{
+    return 'last';
+}
+
 =pod
 
 =head2 pre_modify($dnp, $listp)
@@ -178,6 +189,10 @@ sub _do_modify
             }
 
             if (defined($rule->{match}) && !$match) {
+                if ("$action: $attr" =~ /$rule->{match}/i) {
+                    $match = 1;
+                }
+
                 foreach my $value (@values) {
                     if ("$attr: $value" =~ /$rule->{match}/i) {
                         $match = 1;
