@@ -187,12 +187,17 @@ sub search
         return (LDAP_FILTER_ERROR, ());
     }
 
-    if (!$sizeLim) {
-        $sizeLim = -1;
-    }
+    my $retry = 0;
+    while (1) {
+        if (!$self->_getConnect()) {
+            last;
+        }
 
-    if ($self->_getConnect()) {
-        return (LDAP_SERVER_DOWN, ());
+        if ($retry >= $conf->{connection}->{retry}[0]) {
+            return (LDAP_SERVER_DOWN, ());
+        }
+        sleep $conf->{connection}->{interval}[0];
+        $retry++;
     }
 
     my ($rc, @objs) = $self->_searchObjects($base, $scope);
@@ -265,11 +270,21 @@ sub compare
 {
     my $self = shift;
     my ($dn, $avaStr) = @_;
+    my $conf = $self->{_config};
 
     my ($attr, $val) = split(/=/, $avaStr);
 
-    if ($self->_getConnect()) {
-        return LDAP_SERVER_DOWN;
+    my $retry = 0;
+    while (1) {
+        if (!$self->_getConnect()) {
+            last;
+        }
+
+        if ($retry >= $conf->{connection}->{retry}[0]) {
+            return (LDAP_SERVER_DOWN, ());
+        }
+        sleep $conf->{connection}->{interval}[0];
+        $retry++;
     }
 
     my ($rc, $obj, $pkey) = $self->_getObject($dn);
@@ -313,9 +328,19 @@ sub modify
 {
     my $self = shift;
     my ($dn, @list) = @_;
+    my $conf = $self->{_config};
 
-    if ($self->_getConnect()) {
-        return LDAP_SERVER_DOWN;
+    my $retry = 0;
+    while (1) {
+        if (!$self->_getConnect()) {
+            last;
+        }
+
+        if ($retry >= $conf->{connection}->{retry}[0]) {
+            return (LDAP_SERVER_DOWN, ());
+        }
+        sleep $conf->{connection}->{interval}[0];
+        $retry++;
     }
 
     my ($rc, $obj, $pkey) = $self->_getObject($dn);
@@ -339,16 +364,25 @@ sub add
 {
     my $self = shift;
     my ($dn, $entryStr) = @_;
+    my $conf = $self->{_config};
 
     # check rdn's value
     my ($rdn, $rdn_val) = ($dn =~ /^([^=]+)=([^,]+),/);
-    $rdn_val =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
     if ($entryStr !~ /^$rdn: $rdn_val$/mi) {
         return LDAP_NAMING_VIOLATION;
     }
 
-    if ($self->_getConnect()) {
-        return LDAP_SERVER_DOWN;
+    my $retry = 0;
+    while (1) {
+        if (!$self->_getConnect()) {
+            last;
+        }
+
+        if ($retry >= $conf->{connection}->{retry}[0]) {
+            return (LDAP_SERVER_DOWN, ());
+        }
+        sleep $conf->{connection}->{interval}[0];
+        $retry++;
     }
 
     my ($rc, $obj, $pkey) = $self->_getObject($dn);
@@ -418,8 +452,17 @@ sub delete
     my ($dn) = @_;
     my $conf = $self->{_config};
 
-    if ($self->_getConnect()) {
-        return LDAP_SERVER_DOWN;
+    my $retry = 0;
+    while (1) {
+        if (!$self->_getConnect()) {
+            last;
+        }
+
+        if ($retry >= $conf->{connection}->{retry}[0]) {
+            return (LDAP_SERVER_DOWN, ());
+        }
+        sleep $conf->{connection}->{interval}[0];
+        $retry++;
     }
 
     my ($rc, $obj, $pkey) = $self->_getObject($dn);
@@ -645,6 +688,17 @@ sub _checkConfig
         $conf->{mbcode}[0] = 'utf8';
     }
 
+    # connection
+    if (!defined($conf->{connection}->{retry})
+        || $conf->{connection}->{retry}[0] !~ /[0-9]*/) {
+        $conf->{connection}->{retry}[0] = 0;
+    }
+    if (!defined($conf->{connection}->{interval})
+        || $conf->{connection}->{interval}[0] !~ /[0-9]*/) {
+        $conf->{connection}->{interval}[0] = 10;
+    }
+
+    # data object
     if (defined($conf->{object})) {
         $self->{object} = {};
         foreach my $oname (keys %{$conf->{object}}) {
@@ -669,13 +723,12 @@ sub _checkConfig
                 $self->log(level => 'alert', message => "$oname object container entry is invalid");
                 return 1;
             } else {
-                $oconf->{container}[0]->{rdn}[0] =~ tr/A-Z/a-z/;
-                my $crdn = $oconf->{container}[0]->{rdn}[0];
+                my $crdn = lc($oconf->{container}[0]->{rdn}[0]);
                 my $coname = $crdn.'_container';
                 if (!defined($self->{object}{$coname})) {
                     $self->{object}{$coname}->{entrystr} = $entry;
                     # normalize suffix
-                    ($self->{object}{$coname}->{suffix} = "$crdn,".$self->{suffix}) =~ tr/A-Z/a-z/;
+                    $self->{object}{$coname}->{suffix} = lc("$crdn,".$self->{suffix});
                 }
                 # set child object
                 if (!defined($self->{object}{$coname}->{child})) {
@@ -713,14 +766,6 @@ sub _checkConfig
                 $self->log(level => 'alert', message => "rdn of $oname object is invalid");
                 return 1;
             }
-
-            foreach my $si (@{$oconf->{strginfo}}) {
-                # check type of value
-                if (defined($si->{value}) && !ref($si->{value}[0])) {
-                    $self->log(level => 'alert', message => "type of strginfo value doesn't exist");
-                    return 1;
-                }
-            }
         }
 
         # check container's link
