@@ -156,6 +156,8 @@ sub _objSearch
             chop;
             my @data = split(/$dlm/);
             my $entry;
+            my $rdn_val = $data[$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]];
+            $rdn_val =~ s/ *$//;
 
             # check the number of returned entries
             if ($sizeLim >= 0 && @match_entries == $sizeLim) {
@@ -173,6 +175,7 @@ sub _objSearch
             }
 
             # get all values of the entry
+            my $rdn = "$oconf->{rdn}[0]=$rdn_val";
             foreach my $oc (@{$oconf->{oc}}) {
                 $entry = $entry."objectclass: $oc\n";
             }
@@ -195,10 +198,9 @@ sub _objSearch
                 }
             }
 
+            $rdn = decode($conf->{mbcode}[0], $rdn);
             $entry = decode($conf->{mbcode}[0], $entry);
 
-            my ($rdn_val) = ($entry =~ /^$oconf->{rdn}[0]: (.*)$/mi);
-            my $rdn = "$oconf->{rdn}[0]=$rdn_val";
             $entry = "dn: $rdn,$suffix\n$entry";
 
             # parse filter
@@ -234,10 +236,9 @@ sub _objModify
     my $rc = LDAP_SUCCESS;
 
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
-    $rdn_val =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
 
     # multibyte string
-    $rdn_val = encode($conf->{mbcode}[0], $rdn_val);
+    $rdn_val = encode($conf->{mbcode}[0], decode('utf8', $rdn_val));
 
     my $lock;
     my $file;
@@ -336,10 +337,8 @@ sub _objModify
 
                 if($action eq "ADD") {
                     # check whether the value already exists
-                    for (my $i = 0; $i < @values; $i++) {
-                        my $value = $values[0];
-                        $value =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
-                        if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$value *$valdlm/i) {
+                    foreach my $val (@values) {
+                        if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$val *$valdlm/i) {
                             $rc = LDAP_TYPE_OR_VALUE_EXISTS;
                             last DO;
                         }
@@ -352,12 +351,10 @@ sub _objModify
                 } elsif($action eq "DELETE") {
                     if (@values && $values[0]) {
                         # check whether the value exists
-                        for (my $i = 0;  $i < @values; $i++) {
-                            my $value = $values[0];
-                            $value =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
-                            if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$value *$valdlm/i) {
+                        foreach my $val (@values) {
+                            if ("$valdlm$data[$coln]$valdlm" =~ /$valdlm *$val *$valdlm/i) {
                                 my $str = "$valdlm$data[$coln]$valdlm";
-                                $str =~ s/$valdlm$value$valdlm/$valdlm/i;
+                                $str =~ s/$valdlm$val$valdlm/$valdlm/i;
                                 ($data[$coln]) = ($str =~ /^$valdlm(.*)$valdlm$/);
                             } else {
                                 $rc = LDAP_NO_SUCH_ATTRIBUTE;
@@ -409,7 +406,6 @@ sub _objAdd
     my $rc = LDAP_SUCCESS;
 
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
-    $rdn_val =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
 
     # multibyte string
     $rdn_val = encode($conf->{mbcode}[0], decode('utf8', $rdn_val));
@@ -543,7 +539,6 @@ sub _objDelete
     my $rc = LDAP_NO_SUCH_OBJECT;
 
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
-    $rdn_val =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
 
     # multibyte string
     $rdn_val = encode($conf->{mbcode}[0], decode('utf8', $rdn_val));
